--local base = _G
local print = base.print
local pairs = base.pairs
local ipairs = base.ipairs
local type = base.type
local unpack = base.unpack
local tostring = base.tostring
local debug = base.debug
local setmetatable = base.setmetatable
local getmetatable = base.getmetatable
local loadstring = base.loadstring
local loadfile = base.loadfile
local setfenv=base.setfenv
local string = base.string
local table = base.table
local io = base.io
local math = base.math
local lfs = base.lfs
local unitsList = base.panel_units_list
local MissionModule = base.require('me_mission')
local U = base.require('me_utilities')
local minizip = base.require('minizip')
local Options = base.require('me_options')

----------------------------------------------------------
--
--    159th_Ajax Mission Editor 'Run Script' routines
--
----------------------------------------------------------

-- get options from file
print("Entering special script")
local me_mods_options = {}
local f = loadfile(base.mainPath.."modules/me_mods_options.lua")
local env = {}
if f then
    base.setfenv(f,env)
    f()
    me_mods_options = env.me_mods_options
end
local runScriptOptions = {}
if me_mods_options.runScriptOptions then
    runScriptOptions = me_mods_options.runScriptOptions
else
    runScriptOptions["printUnits"] = false
    runScriptOptions["printCoalitions"] = false
    runScriptOptions["printGroupSummary"] = false
    runScriptOptions["printFlyables"] = false
end

me_mods = {}

me_mods.unitCompare = function(unit1,unit2)
    -- custom table.sort function for units list
    -- sorts by side (desc), group (asc), unit (asc)
    if unit1.coalition == unit2.coalition then
        if unit1.group == unit2.group then
            return unit1.name < unit2.name
        else
            return unit1.group < unit2.group
        end
    else
        return unit1.coalition > unit2.coalition    -- reverse order
    end
end

-- adapted from ED ME function
me_mods.createGroupByNameTable = function()
    local group_by_name = {}
    
    for r,group in pairs(MissionModule.group_by_name) do
        if group then 
            table.insert(group_by_name,group) 
        end
    end
    
    local sortGroupByName = function(group1, group2)
        -- sorts by coalition (desc), groupname (asc)
        if group1.boss.boss.name == group2.boss.boss.name then
            return group1.name < group2.name
        else
            return group1.boss.boss.name > group2.boss.boss.name
        end
--        return group1.name < group2.name
    end
    
    table.sort(group_by_name, sortGroupByName)
    
    return group_by_name
end

me_mods.getUnitsTable = function()
    -- build units table with all needed info in one place
    local _tmp = {}
    for unit_id, unit_data in pairs(MissionModule.unit_by_id) do
        if type(unit_data) == 'table' then
            local _n = #_tmp + 1
            _tmp[_n] = {}
            _tmp[_n].index           =    unit_data.index
            _tmp[_n].livery_id       =    unit_data.livery_id or ""
            _tmp[_n].skill           =    unit_data.skill or ""
            _tmp[_n].group           =    unit_data.boss.name
            _tmp[_n].group_type      =    unit_data.boss.type
            _tmp[_n].group_hidden    =    unit_data.boss.hidden or false
            _tmp[_n].country         =    unit_data.boss.boss.name
            _tmp[_n].coalition       =    unit_data.boss.boss.boss.name
            _tmp[_n].type            =    unit_data.type
            _tmp[_n].unitId          =    unit_data.unitId
            _tmp[_n].name            =    unit_data.name
            _tmp[_n].onboard_num     =    unit_data.onboard_num or ""
            _tmp[_n].callsign        =    unit_data.callsign or ""
            _tmp[_n].group_task      =    unit_data.boss.task or ""
        end
    end
    return _tmp
end

me_mods.getUnitInfo = function()

    -- To do: re-work to eliminate repetition

    local _tmp = me_mods.getUnitsTable()
    table.sort(_tmp,me_mods.unitCompare)

    print("    AIRCRAFT")
    print(" ")
    local s = string.format("    %-7s%-15s%-27s%-27s%-37s%-12s%-34s%-8s%-5s%-15s%s","SIDE","COUNTRY","GROUP","UNIT","TYPE","SKILL","SKIN","HIDDEN","BORT","CALLSIGN","TASK")
    print(s)
    print(" ")
    for i,v in pairs(_tmp) do
        if i then
            if v.group_type == 'plane' or v.group_type == 'helicopter' then
                s = string.format("    %-7s%-15s%-27s%-27s%-37s%-12s%-34s%-8s%-5s%-15s%s",v.coalition,v.country,v.group,v.name,v.type,v.skill,string.sub(v.livery_id,1,33),tostring(v.group_hidden),v.onboard_num,v.callsign,v.group_task)
                print(s)
            end
        end
    end
    print(" ")
    
    print("    SHIPS")
    print(" ")
    local s = string.format("    %-7s%-15s%-27s%-27s%-37s%-12s%-8s%s","SIDE","COUNTRY","GROUP","UNIT","TYPE","SKILL","HIDDEN","TASK")
    print(s)
    print(" ")
    for i,v in pairs(_tmp) do
        if i then
            if v.group_type == 'ship' then
                s = string.format("    %-7s%-15s%-27s%-27s%-37s%-12s%-8s%s",v.coalition,v.country,v.group,v.name,v.type,v.skill,tostring(v.group_hidden),v.group_task)
                print(s)
            end
        end
    end
    print(" ")
    
    print("    VEHICLES")
    print(" ")
    local s = string.format("    %-7s%-15s%-27s%-31s%-37s%-12s%-8s%s","SIDE","COUNTRY","GROUP","UNIT","TYPE","SKILL","HIDDEN","TASK")
    print(s)
    print(" ")
    for i,v in pairs(_tmp) do
        if i then
            if v.group_type == 'vehicle' then
                s = string.format("    %-7s%-15s%-27s%-31s%-37s%-12s%-8s%s",v.coalition,v.country,v.group,v.name,v.type,v.skill,tostring(v.group_hidden),v.group_task)
                print(s)
            end
        end
    end
    print(" ")
    
    print("    STATICS")
    print(" ")
    local s = string.format("    %-7s%-15s%-27s%-31s%-37s%-8s","SIDE","COUNTRY","GROUP","UNIT","TYPE","HIDDEN")
    print(s)
    print(" ")
    for i,v in pairs(_tmp) do
        if i then
            if v.group_type == 'static' then
                s = string.format("    %-7s%-15s%-27s%-31s%-37s%-8s",v.coalition,v.country,v.group,v.name,v.type,tostring(v.group_hidden))
                print(s)
            end
        end
    end
    print(" ")
end

me_mods.getGroupInfo = function(printSummary,printList)

    local rowcount = base.panel_units_list.grid:getRowCount()
    local w, group, groupname, country, grouptype, side, skill
    local lvid, utype, bort, callsign
    local nplane = 0
    local nhelicopter = 0
    local nstatic = 0
    local nvehicle = 0
    local nship = 0
    local grp = me_mods.createGroupByNameTable()
    
    if printSummary then
        for i,v in pairs(grp) do
            grouptype = v.type
            if grouptype == 'plane' then nplane = nplane + 1 end
            if grouptype == 'helicopter' then nhelicopter = nhelicopter + 1 end
            if grouptype == 'static' then nstatic = nstatic + 1 end
            if grouptype == 'vehicle' then nvehicle = nvehicle + 1 end
            if grouptype == 'ship' then nship = nship + 1 end
        end
        local ntotal = nplane + nhelicopter + nstatic + nvehicle + nship
        print("   ")
        print(string.format("    %-17s : %+3s","Total groups",ntotal))
        print(string.format("    %-17s : %+3s","Ship groups",nship))
        print(string.format("    %-17s : %+3s","Plane groups",nplane))
        print(string.format("    %-17s : %+3s","Helicopter groups",nhelicopter))
        print(string.format("    %-17s : %+3s","Vehicle groups",nvehicle))
        print(string.format("    %-17s : %+3s","Static groups",nstatic))
    end

    if printList then
        print("  ")
        print(string.format("    %-8s%-36s%-15s%-15s%+4s       %-43s%s","SIDE","GROUP","COUNTRY","TYPE","BORT","SKIN","CALLSIGN"))
        print(" ")
        for i,v in pairs(grp) do
            group = v
            groupname = group.name
            grouptype = group.type
            country = group.boss.name
            side = group.boss.boss.name
            lvid = group.units[1].livery_id
            utype = group.units[1].type
            skill = group.units[1].skill
            bort = group.units[1].onboard_num
            callsign = group.units[1].callsign
            if bort == nil then bort = "" end
            if utype == nil then utype = "" end
            if skill == nil then skill = "" end
            if lvid == nil then lvid = "" end
            if skill == 'Client' and (grouptype == 'plane' or grouptype == 'helicopter') then
                local s
                s = string.format("    %-8s%-36s%-15s%-15s%+4s       %-43s%s",side,groupname,country,utype,bort,lvid,callsign)
                print(s)
            end
        end
    end
    if printSummary or printList then print("     ") end
end

me_mods.getCoalitionInfo = function()
    local coa = MissionModule.mission.coalitions
    print("       ")
    print("    COALITIONS")
    print(" ")
    for i in pairs(coa) do
        for j,v in pairs(coa[i]) do
            local s = string.format("    %-8s%+2s     %s",i,j,v)
            print(s)
        end
    end
    print(" ")
end

me_mods.reassignCallsigns = function()

    local function tsort(u1,u2)
        if u1.boss.boss.boss.name == u2.boss.boss.boss.name then    -- coalition
            if u1.boss.name == u2.boss.name then    -- group
                return u1.index < u2.index
            else
                return u1.boss.name < u2.boss.name
            end
        else
            return u1.boss.boss.boss.name > u2.boss.boss.boss.name
        end
    end
    local _tmp = {}
	
    for i,v in pairs(MissionModule.unit_by_name) do
        table.insert(_tmp,v)
    end
    table.sort(_tmp,tsort)
    local n = 100
    for unit,unit_data in pairs(_tmp) do
        if type(unit_data) == 'table' then
            if (unit_data.boss.type == 'helicopter' or unit_data.boss.type == 'plane') then
                if me_mods_options.reassignEasternCallsigns and not U.isWesternCountry(unit_data.boss.boss.name) then
                    n = n + 1
                    local oldn = unit_data.callsign
                    unit_data.callsign = n
                end
            end
        end
    end

end


---------------------------------------------------------------------------------------------------------------------------------------------


--[[
me_mods.getMissionFromFile = function(missionFileName)
    --from me_openfile.lua
    local fname = missionFileName
    print("Getting mission file...")
    if minizip ~= nil then
        --print(type(minizip))
        print("minizip is not nil.")
    end
    local zipFile = minizip.unzOpen(fname, 'rb')
    print("Loaded mission file...")
    if not zipFile then
        print("Mission file could not be loaded.")
        return ''
    end
    print("Loaded mission file...")
    local misStr
    if zipFile:unzLocateFile('mission') then
        misStr = zipFile:unzReadAllCurrentFile()
    end
    zipFile:unzClose()
    local fun = loadstring(misStr)
    local env = { }
    setfenv(fun, env)
    fun()

    local mission = env.mission
    
    for i,v in ipairs(mission.trig.actions)
        print("["..i.."]  "..v)
    end
end
--]]


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.setSkin = function(coa,ac,skin)
    coa = string.upper(coa)
    ac = string.upper(ac)
    for unit_id, unit_data in pairs(MissionModule.unit_by_id) do
		print("unitID: "..unit_id.." name: "..(unit_data.name or "nil").." type: "..(unit_data.type or "nil").." skill: "..(unit_data.skill or "nil").." skin: "..(unit_data.skin or "nil"))
        if type(unit_data) == 'table' then
            if string.upper(unit_data.boss.boss.boss.name) == coa then
                if string.upper(unit_data.type or "") == ac then
                    if ac == 'IL-78M' or ac == 'A-50' or string.upper(unit_data.skill or "") == 'CLIENT' then
                        unit_data.livery_id = skin
                    end
                end
            end
        end
    end
end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.setFreq = function(baseFreq, acType, groupSize, freqIncrement)

    local function tsort(g1,g2)
        if g1.boss.boss.name == g2.boss.boss.name then    -- coalition name
			return g1.name < g2.name -- group name
        else
			return g1.boss.boss.name > g2.boss.boss.name -- red gets put at top of list
        end
    end

	local clients = {}
	clients['red'] = {}
	clients['blue'] = {}
    
    for unit_id, unit_data in pairs(MissionModule.unit_by_id) do
        if type(unit_data) == 'table' then
            if string.upper(unit_data.type or "") == string.upper(acType) and string.upper(unit_data.skill or "") == 'CLIENT' then
				table.insert(clients[string.lower(unit_data.boss.boss.boss.name)],unit_data.boss) -- group table
			end
		end
	end

	table.sort(clients['red'],tsort)
	table.sort(clients['blue'],tsort)

	for coa,coa_data in pairs(clients) do
		local cntr = 0
		for group_id, group_data in pairs(coa_data) do
			local mult = math.floor(cntr/groupSize)
			group_data.frequency = baseFreq + mult * freqIncrement
			cntr = cntr + 1
			print("setFreq "..coa.." "..group_data.units[1].type)
		end
	end

end


---------------------------------------------------------------------------------------------------------------------------------------------


me_mods.setOptions = function(mOptions)

    if mOptions == nil then mOptions = {} end
    Options.setForcedOptions(mOptions)

end


---------------------------------------------------------------------------------------------------------------------------------------------


if me_mods_options.reassignCallsigns then me_mods.reassignCallsigns() end
if runScriptOptions.printCoalitions then me_mods.getCoalitionInfo() end
if runScriptOptions.printGroupSummary or runScriptOptions.printFlyables then me_mods.getGroupInfo(runScriptOptions.printGroupSummary,runScriptOptions.printFlyables) end
if runScriptOptions.printUnits then me_mods.getUnitInfo() end


---------------------------------------------------------------------------------------------------------------------------------------------
--
--    me_mods.setSkin( coalition, aircraftType, skinName )
--
--    Sets skins for all *client* aircraft of the specified type and coalition. Exceptions are made for the IL-78M and A-50.
--    All skins for a given type are set to the same skin. There is no 'easy' way to programmatically allow for mixed skins.
--
--    coalition      =   'RED' or 'BLUE'
--    aircraftType   =   'Ka-50', 'UH-1H', 'Mi-8MTV2', etc. Must be the exact name used by DCS (see table below).
--    skinName       =    folder name of the skin files. Note: this is different from the description that appears in the ME drop-down dialog box.
--
--    Client aircraft names having squad skins
--
--   "A-10C"
--   "A-50"       * AI exception
--   "F-15C"
--   "IL-78M"     * AI exception
--   "MiG-29S"
--   "P-51D"
--   "Su-25"
--   "Su-25T"
--   "Su-27"
--   "Su-33"
--   "Ka-50"
--   "UH-1H"
--   "Mi-8MTV2"   * Note: "Mi-8MTV2" -- not the usual DCS designation ("Mi-8MT") -- is required here.

---[[
    me_mods.setSkin(  "RED"  ,  "Ka-50"     ,   "Russia Standard Army (Worn)"         )
    me_mods.setSkin(  "RED"  ,  "UH-1H"     ,   "159th GAR splinter"                  )
--  me_mods.setSkin(  "RED"  ,  "UH-1H"     ,   "159th GAR"                           )
    me_mods.setSkin(  "RED"  ,  "Su-25"     ,   "159th SU-25 Splinter Camo"           )
--  me_mods.setSkin(  "RED"  ,  "Su-25"     ,   "159th SU-25"                         )
    me_mods.setSkin(  "RED"  ,  "Su-25T"    ,   "159th GAR"                           )
--  me_mods.setSkin(  "RED"  ,  "Su-25T"    ,   "159th GAR Salmon"                    )
--  me_mods.setSkin(  "RED"  ,  "Su-25T"    ,   "159th GAR Splinter Camo W"           )
    me_mods.setSkin(  "RED"  ,  "A-10C"     ,   "159th GAR A-10C"                     )
    me_mods.setSkin(  "RED"  ,  "Mi-8MTV2"  ,   "159th_GAR Splinter"                  )
--  me_mods.setSkin(  "RED"  ,  "Mi-8MTV2"  ,   "159th_GAR curvy"                     )
--  me_mods.setSkin(  "RED"  ,  "Mi-8MTV2"  ,   "159th_GAR curvy matt"                )
    me_mods.setSkin(  "RED"  ,  "IL-78M"    ,   "159th GAR"                           )
    me_mods.setSkin(  "RED"  ,  "MiG-29S"   ,   "115th guards regiment, termez ab"    )
    me_mods.setSkin(  "RED"  ,  "Su-27"     ,   "159th GAR splinter"                  )
    me_mods.setSkin(  "RED"  ,  "Su-33"     ,   "standard-2 navy"                     )
    me_mods.setSkin(  "RED"  ,  "P-51D"     ,   "Russia 159th GAR"                    )
    me_mods.setSkin(  "RED"  ,  "A-50"      ,   "159th GAR"                           )
    me_mods.setSkin(  "RED"  ,  "F-15C"     ,   "159th Agressor"                      )
    --me_mods.setSkin(  "BLUE" ,  "F-15C"     ,   "159th Agressor"                      )
--]]
    
---------------------------------------------------------------------------------------------------------------------------------------------
--
-- me_mods.setFreq(baseFreq, acType, groupSize, freqIncrement)
--
-- The default frequency for Ka-50s, 124.000, is now bugged.
-- This function was created to easily change the frequency of all Ka-50s in the mission.
-- However, its use is not restricted to Ka-50s -- the frequency for any a/c type can be set.
--
-- baseFreq      = starting frequency
-- acType        = aircraft type (see above for listing of a/c types)
-- groupSize     = number of units to get the same frequency
-- freqIncrement = the amount to add to the base frequency after every nth unit where n = 'groupSize'
--
-- Each coalition is treated separately.
-- Operates on client units only.

	me_mods.setFreq( 127.5 , 'Ka-50' , 4 , 0.5 )
	me_mods.setFreq( 251   , 'UH-1H' , 4 , 1   )
   
---------------------------------------------------------------------------------------------------------------------------------------------
--
-- me_mods.setOptions(mOptions)
--
-- Sets/clears mission options.
--
-- mOptions  =  Lua table containing the forced options defined below.
--
-- Comment out each line you do *not* want to enforce.
-- Uncomment and set the value for each one you *do* want to enforce.

  local mOptions = {}
  
  mOptions.padlock                 = false
  mOptions.easyRadar               = false
  mOptions.scenes                  = "high"                   --  "low", "medium", "high"
  mOptions.birds                   = 0                        --   numeric: 0 to 1000
  mOptions.optionsView             = "optview_onlyallies"     --  "optview_onlymap", "optview_myaircraft", "optview_allies", "optview_onlyallies", "optview_all"
  mOptions.permitCrash             = false                        
  mOptions.immortal                = false                        
  mOptions.easyCommunication       = false                        
  mOptions.easyFlight              = false                        
  mOptions.radio                   = false                        
--mOptions.tips                    = false                    --   this option will be set to un-forced
  mOptions.accidental_failures     = false                        
  mOptions.fuel                    = false                        
  mOptions.externalViews           = true                         
  mOptions.geffect                 = "realistic"              --  "none", "reduced", "realistic"
  mOptions.civTraffic              = ""                       --  "", "low", "medium", "high"
  mOptions.weapons                 = false
  mOptions.miniHUD                 = false
  mOptions.labels                  = false

  
  me_mods.setOptions(mOptions)

   

---------------------------------------------------------------------------------------------------------------------------------------------

-- testing to see if mission file can be loaded directly from disk, unzipped, manipulated, rezipped, then saved

--me_mods.getMissionFromFile("D:/Users/AJ/Saved Games/DCS.openbeta/Missions/current public/KrasnodarKraze_v4.miz")

---------------------------------------------------------------------------------------------------------------------------------------------




print("End special script.")
